// Copyright Epic Games, Inc. All Rights Reserved.

#include "MonolithCharacter.h"

#include "Animation/AnimInstance.h"
#include "Components/CapsuleComponent.h"
#include "Components/SkeletalMeshComponent.h"
#include "Engine/LocalPlayer.h"
#include "GameFramework/CharacterMovementComponent.h"

#include "EnhancedInputComponent.h"
#include "EnhancedInputSubsystems.h"
#include "InputActionValue.h"

DEFINE_LOG_CATEGORY(LogTemplateCharacter);

//////////////////////////////////////////////////////////////////////////
// AMonolithCharacter

void AMonolithCharacter::StartRechargingDash()
{
	if (DashRechargeHandle.IsValid()) return;
	GetWorld()->GetTimerManager().SetTimer(DashRechargeHandle, this
		, &AMonolithCharacter::DashRecharged, DashRechargeTime);
}

void AMonolithCharacter::DashRecharged()
{
	DashesRemaining++;
	UpdateDashUI(DashesRemaining);
	DashRechargeHandle.Invalidate();
	if (DashesRemaining < 4)
	{
		StartRechargingDash();
	}
}

AMonolithCharacter::AMonolithCharacter()
{
	// Set size for collision capsule
	GetCapsuleComponent()->InitCapsuleSize(55.f, 96.0f);
}

void AMonolithCharacter::BeginPlay()
{
	// Call the base class  
	Super::BeginPlay();

	Mesh1P   = Cast<USkeletalMeshComponent>(GetComponentByClass(USkeletalMeshComponent::StaticClass()));
	CharMove = Cast<UCharacterMovementComponent>(GetComponentByClass(UCharacterMovementComponent::StaticClass()));
	if (CharMove) WalkSpeed = CharMove->MaxWalkSpeed;
}

void AMonolithCharacter::Tick(float DeltaSeconds)
{
	Super::Tick(DeltaSeconds);
	if (CharMove == nullptr) return;
	CharMove->IsFalling() ? MoveState.Remove(GROUNDED) : MoveState.Add(GROUNDED);
	MoveState.VelocityMag = CharMove->Velocity.Length();
	MoveState.VelocityMag < VelocityTolerance ? MoveState.Add(STILL) : MoveState.Remove(STILL);
}

//////////////////////////////////////////////////////////////////////////// Input

void AMonolithCharacter::SetupPlayerInputComponent(UInputComponent* PlayerInputComponent)
{	
	// Set up action bindings
	if (UEnhancedInputComponent* EnhancedInputComponent = Cast<UEnhancedInputComponent>(PlayerInputComponent))
	{
		// Jumping
		EnhancedInputComponent->BindAction(JumpAction, ETriggerEvent::Started, this, &ACharacter::Jump);
		EnhancedInputComponent->BindAction(JumpAction, ETriggerEvent::Completed, this, &ACharacter::StopJumping);

		// Moving
		EnhancedInputComponent->BindAction(MoveAction, ETriggerEvent::Triggered, this, &AMonolithCharacter::Move);
		EnhancedInputComponent->BindAction(MoveAction, ETriggerEvent::Completed, this, &AMonolithCharacter::StopMove);

		// Sprinting
		EnhancedInputComponent->BindAction(SprintAction, ETriggerEvent::Triggered, this, &AMonolithCharacter::Sprint);
		EnhancedInputComponent->BindAction(SprintAction, ETriggerEvent::Completed, this, &AMonolithCharacter::StopSprint);

		// Dashing
		EnhancedInputComponent->BindAction(HorDashAction, ETriggerEvent::Triggered, this, &AMonolithCharacter::HorizontalDash);
		EnhancedInputComponent->BindAction(VerDashAction, ETriggerEvent::Triggered, this, &AMonolithCharacter::VerticalDash);

		// Looking
		EnhancedInputComponent->BindAction(LookAction, ETriggerEvent::Triggered, this, &AMonolithCharacter::Look);
	}
	else
	{
		UE_LOG(LogTemplateCharacter, Error, TEXT("'%s' Failed to find an Enhanced Input Component! This template is built to use the Enhanced Input system. If you intend to use the legacy system, then you will need to update this C++ file."), *GetNameSafe(this));
	}
}

static void UpdateMoveState(FMoveState& MoveState, int Forwards, int Sideways)
{
	if (Sideways && !Forwards)
	{	// STRAFING
		if (Sideways > 0)
		{
			MoveState.Add(STRAFE_R);
			MoveState.Remove(STRAFE_L);
		}
		else
		{
			MoveState.Add(STRAFE_L);
			MoveState.Remove(STRAFE_R);
		}
		MoveState.Remove(FORWARD);
	}
	else if (Forwards > 0)
	{	// GOING ROUGHLY FORWARD
		MoveState.Add(FORWARD);
		MoveState.Remove(STRAFE_L);
		MoveState.Remove(STRAFE_R);
	}
	else
	{
		MoveState.Remove(FORWARD);
		MoveState.Remove(STRAFE_L);
		MoveState.Remove(STRAFE_R);
	}
}

void AMonolithCharacter::Move(const FInputActionValue& Value)
{
	// input is a Vector2D
	MovementVector = Value.Get<FVector2D>();

	if (Controller != nullptr)
	{
		// add movement 
		AddMovementInput(GetActorForwardVector(), MovementVector.Y);
		AddMovementInput(GetActorRightVector(), MovementVector.X);
		UpdateMoveState(MoveState, MovementVector.Y, MovementVector.X);
	}
}

void AMonolithCharacter::StopMove()
{
	MoveState.Remove(STRAFE_L);
	MoveState.Remove(STRAFE_R);
	MoveState.Remove(FORWARD);
	MoveState.Remove(SPRINT);
}

void AMonolithCharacter::Sprint(const FInputActionValue& Value)
{
	if (CharMove == nullptr || MoveState.Is(SPRINT)) return;

	if (!MoveState.Is(FORWARD)
	 || !MoveState.Is(GROUNDED))
	{
		StopSprint();
	}
	else
	{
		MoveState.Add(SPRINT);
		CharMove->MaxWalkSpeed = SprintSpeed;
	}
}

void AMonolithCharacter::StopSprint()
{
	MoveState.Remove(SPRINT);
	CharMove->MaxWalkSpeed = WalkSpeed;
}

void AMonolithCharacter::HorizontalDash(const FInputActionValue& Value)
{
	if (DashesRemaining <= 0) return;
	DashesRemaining--;
	UpdateDashUI(DashesRemaining);
	FVector Impulse;
	if (MovementVector.Y)
	{
		Impulse = MovementVector.Y > 0
			? GetActorForwardVector() : -GetActorForwardVector();
	}
	else
	{
		Impulse = MovementVector.X > 0
			? GetActorRightVector() : -GetActorRightVector();
	}
	Impulse *= DashImpulse;
	Impulse.Z += DashImpulse * 0.25f;
	CharMove->AddImpulse(Impulse, true);
	StartRechargingDash();
}

void AMonolithCharacter::VerticalDash(const FInputActionValue& Value)
{
	if (DashesRemaining <= 0) return;
	DashesRemaining--;
	UpdateDashUI(DashesRemaining);
	FVector Impulse(0, 0, DashImpulse * 0.55f);
	CharMove->AddImpulse(Impulse, true);
	StartRechargingDash();
}

void AMonolithCharacter::Look(const FInputActionValue& Value)
{
	// input is a Vector2D
	FVector2D LookAxisVector = Value.Get<FVector2D>();

	if (Controller != nullptr)
	{
		// add yaw and pitch input to controller
		AddControllerYawInput(LookAxisVector.X);
		AddControllerPitchInput(LookAxisVector.Y);
	}
}
